@@ -62,6 +62,9 @@ function configure_mistral {
     oslo-config-generator --config-file $MISTRAL_DIR/tools/config/config-generator.mistral.conf --output-file $MISTRAL_CONF_FILE
     iniset $MISTRAL_CONF_FILE DEFAULT debug $MISTRAL_DEBUG
 
+    MISTRAL_POLICY_FILE=$MISTRAL_CONF_DIR/policy.json
+    cp $MISTRAL_DIR/etc/policy.json $MISTRAL_POLICY_FILE
+
     # Run all Mistral processes as a single process
     iniset $MISTRAL_CONF_FILE DEFAULT server all
 
@@ -89,9 +92,16 @@ function configure_mistral {
     # Configure action execution deletion policy
     iniset $MISTRAL_CONF_FILE api allow_action_execution_deletion True
 
+    # Path of policy.json file.
+    iniset $MISTRAL_CONF oslo_policy policy_file $MISTRAL_POLICY_FILE
+
     if [ "$LOG_COLOR" == "True" ] && [ "$SYSLOG" == "False" ]; then
         setup_colorized_logging $MISTRAL_CONF_FILE DEFAULT tenant user
     fi
+
+    if [ "$MISTRAL_RPC_IMPLEMENTATION" ]; then
+        iniset $MISTRAL_CONF_FILE DEFAULT rpc_implementation $MISTRAL_RPC_IMPLEMENTATION
+    fi
 }
 
 
@@ -105,7 +115,7 @@ function init_mistral {
 
 # install_mistral - Collect source and prepare
 function install_mistral {
-    setup_develop $MISTRAL_DIR -e
+    setup_develop $MISTRAL_DIR
 
     # installing python-nose.
     real_install_package python-nose
@@ -118,7 +128,7 @@ function install_mistral {
 
 function _install_mistraldashboard {
     git_clone $MISTRAL_DASHBOARD_REPO $MISTRAL_DASHBOARD_DIR $MISTRAL_DASHBOARD_BRANCH
-    setup_develop $MISTRAL_DASHBOARD_DIR -e
+    setup_develop $MISTRAL_DASHBOARD_DIR
     ln -fs $MISTRAL_DASHBOARD_DIR/_50_mistral.py.example $HORIZON_DIR/openstack_dashboard/local/enabled/_50_mistral.py
 }
 
@@ -130,21 +140,32 @@ function install_mistral_pythonclient {
         if [ ! "$tags" = "" ]; then
             git --git-dir=$MISTRAL_PYTHONCLIENT_DIR/.git tag -d $tags
         fi
-        setup_develop $MISTRAL_PYTHONCLIENT_DIR -e
+        setup_develop $MISTRAL_PYTHONCLIENT_DIR
     fi
 }
 
 
 # start_mistral - Start running processes, including screen
 function start_mistral {
-    screen_it mistral "cd $MISTRAL_DIR && $MISTRAL_BIN_DIR/mistral-server --config-file $MISTRAL_CONF_DIR/mistral.conf"
+    if is_service_enabled mistral-api && is_service_enabled mistral-engine && is_service_enabled mistral-executor && is_service_enabled mistral-event-engine ; then
+        echo_summary "Installing all mistral services in separate processes"
+        run_process mistral-api "$MISTRAL_BIN_DIR/mistral-server --server api --config-file $MISTRAL_CONF_DIR/mistral.conf"
+        run_process mistral-engine "$MISTRAL_BIN_DIR/mistral-server --server engine --config-file $MISTRAL_CONF_DIR/mistral.conf"
+        run_process mistral-executor "$MISTRAL_BIN_DIR/mistral-server --server executor --config-file $MISTRAL_CONF_DIR/mistral.conf"
+        run_process mistral-event-engine "$MISTRAL_BIN_DIR/mistral-server --server event-engine --config-file $MISTRAL_CONF_DIR/mistral.conf"
+    else
+        echo_summary "Installing all mistral services in one process"
+        run_process mistral "$MISTRAL_BIN_DIR/mistral-server --server all --config-file $MISTRAL_CONF_DIR/mistral.conf"
+    fi
 }
 
 
 # stop_mistral - Stop running processes
 function stop_mistral {
     # Kill the Mistral screen windows
-    screen -S $SCREEN_NAME -p mistral -X kill
+    for serv in mistral mistral-api mistral-engine mistral-executor mistral-event-engine; do
+        stop_process $serv
+    done
 }
 
 
