@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
-#
 # Copyright 2013 - Mirantis, Inc.
+# Copyright 2016 - Brocade Communications Systems, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -16,6 +15,7 @@
 
 from oslo_config import cfg
 import oslo_middleware.cors as cors_middleware
+import osprofiler.web
 import pecan
 
 from mistral.api import access_control
@@ -23,6 +23,7 @@ from mistral import config as m_config
 from mistral import context as ctx
 from mistral import coordination
 from mistral.db.v2 import api as db_api_v2
+from mistral.engine.rpc_backend import rpc
 from mistral.services import periodic
 
 
@@ -45,13 +46,15 @@ def get_pecan_config():
 def setup_app(config=None):
     if not config:
         config = get_pecan_config()
+
     m_config.set_config_defaults()
 
     app_conf = dict(config.app)
 
     db_api_v2.setup_db()
 
-    periodic.setup()
+    if not app_conf.pop('disable_cron_trigger_thread', False):
+        periodic.setup()
 
     coordination.Service('api_group').register_membership()
 
@@ -65,8 +68,17 @@ def setup_app(config=None):
     # Set up access control.
     app = access_control.setup(app)
 
+    # Set up RPC related flags in config
+    rpc.get_transport()
+
+    # Set up profiler.
+    if cfg.CONF.profiler.enabled:
+        app = osprofiler.web.WsgiMiddleware(
+            app,
+            hmac_keys=cfg.CONF.profiler.hmac_keys,
+            enabled=cfg.CONF.profiler.enabled
+        )
+
     # Create a CORS wrapper, and attach mistral-specific defaults that must be
     # included in all CORS responses.
-    app = cors_middleware.CORS(app, cfg.CONF)
-
-    return app
+    return cors_middleware.CORS(app, cfg.CONF)
