@@ -15,7 +15,6 @@
 
 from mistral.db.v2 import api as db_api
 from mistral.engine import base
-from mistral.engine import rpc
 from mistral import expressions
 from mistral.services import scheduler
 from mistral.utils import wf_trace
@@ -24,8 +23,8 @@ from mistral.workflow import states
 
 import six
 
-_ENGINE_CLIENT_PATH = 'mistral.engine.rpc.get_engine_client'
-_RUN_EXISTING_TASK_PATH = 'mistral.engine.task_handler.run_existing_task'
+_CONTINUE_TASK_PATH = 'mistral.engine.policies._continue_task'
+_COMPLETE_TASK_PATH = 'mistral.engine.policies._complete_task'
 
 
 def _log_task_delay(task_ex, delay_sec):
@@ -180,7 +179,7 @@ class WaitBeforePolicy(base.TaskPolicy):
         policy_context = runtime_context[context_key]
 
         if policy_context.get('skip'):
-            # Unset state 'DELAYED'.
+            # Unset state 'RUNNING_DELAYED'.
             wf_trace.info(
                 task_ex,
                 "Task '%s' [%s -> %s]"
@@ -193,13 +192,14 @@ class WaitBeforePolicy(base.TaskPolicy):
 
         if task_ex.state != states.IDLE:
             policy_context.update({'skip': True})
+
             _log_task_delay(task_ex, self.delay)
 
             task_ex.state = states.RUNNING_DELAYED
 
             scheduler.schedule_call(
                 None,
-                _RUN_EXISTING_TASK_PATH,
+                _CONTINUE_TASK_PATH,
                 self.delay,
                 task_ex_id=task_ex.id,
             )
@@ -228,6 +228,7 @@ class WaitAfterPolicy(base.TaskPolicy):
         task_ex.runtime_context = runtime_context
 
         policy_context = runtime_context[context_key]
+
         if policy_context.get('skip'):
             # Skip, already processed.
             return
@@ -236,17 +237,25 @@ class WaitAfterPolicy(base.TaskPolicy):
 
         _log_task_delay(task_ex, self.delay)
 
-        state = task_ex.state
+        end_state = task_ex.state
+        end_state_info = task_ex.state_info
+
+        # TODO(rakhmerov): Policies probably needs to have tasks.Task
+        # interface in order to change manage task state safely.
         # Set task state to 'DELAYED'.
         task_ex.state = states.RUNNING_DELAYED
+        task_ex.state_info = (
+            'Suspended by wait-after policy for %s seconds' % self.delay
+        )
 
         # Schedule to change task state to RUNNING again.
         scheduler.schedule_call(
-            _ENGINE_CLIENT_PATH,
-            'on_task_state_change',
+            None,
+            _COMPLETE_TASK_PATH,
             self.delay,
-            state=state,
             task_ex_id=task_ex.id,
+            state=end_state,
+            state_info=end_state_info
         )
 
 
@@ -282,13 +291,12 @@ class RetryPolicy(base.TaskPolicy):
         """
         super(RetryPolicy, self).after_task_complete(task_ex, task_spec)
 
-        # TODO(m4dcoder): If the task_ex.executions collection is not called,
+        # TODO(m4dcoder): If the task_ex.action_executions and
+        # task_ex.workflow_executions collection are not called,
         # then the retry_no in the runtime_context of the task_ex will not
         # be updated accurately. To be exact, the retry_no will be one
-        # iteration behind. task_ex.executions was originally called in
-        # get_task_execution_result but it was refactored to use
-        # db_api.get_action_executions to support session-less use cases.
-        action_ex = task_ex.executions  # noqa
+        # iteration behind.
+        ex = task_ex.executions  # noqa
 
         context_key = 'retry_task_policy'
 
@@ -297,9 +305,17 @@ class RetryPolicy(base.TaskPolicy):
             context_key
         )
 
+        wf_ex = task_ex.workflow_execution
+
+        ctx_view = data_flow.ContextView(
+            data_flow.evaluate_task_outbound_context(task_ex),
+            wf_ex.context,
+            wf_ex.input
+        )
+
         continue_on_evaluation = expressions.evaluate(
             self._continue_on_clause,
-            data_flow.evaluate_task_outbound_context(task_ex)
+            ctx_view
         )
 
         task_ex.runtime_context = runtime_context
@@ -339,7 +355,7 @@ class RetryPolicy(base.TaskPolicy):
 
         scheduler.schedule_call(
             None,
-            _RUN_EXISTING_TASK_PATH,
+            _CONTINUE_TASK_PATH,
             self.delay,
             task_ex_id=task_ex.id,
         )
@@ -360,7 +376,7 @@ class TimeoutPolicy(base.TaskPolicy):
 
         scheduler.schedule_call(
             None,
-            'mistral.engine.policies.fail_task_if_incomplete',
+            'mistral.engine.policies._fail_task_if_incomplete',
             self.delay,
             task_ex_id=task_ex.id,
             timeout=self.delay
@@ -399,7 +415,6 @@ class PauseBeforePolicy(base.TaskPolicy):
         task_ex.state = states.IDLE
 
 
-# TODO(rakhmerov): In progress.
 class ConcurrencyPolicy(base.TaskPolicy):
     _schema = {
         "properties": {
@@ -413,6 +428,10 @@ class ConcurrencyPolicy(base.TaskPolicy):
     def before_task_start(self, task_ex, task_spec):
         super(ConcurrencyPolicy, self).before_task_start(task_ex, task_spec)
 
+        # This policy doesn't do anything except validating "concurrency"
+        # property value and setting a variable into task runtime context.
+        # This variable is then used to define how many action executions
+        # may be started in parallel.
         context_key = 'concurrency'
 
         runtime_context = _ensure_context_has_key(
@@ -424,21 +443,35 @@ class ConcurrencyPolicy(base.TaskPolicy):
         task_ex.runtime_context = runtime_context
 
 
-def fail_task_if_incomplete(task_ex_id, timeout):
-    task_ex = db_api.get_task_execution(task_ex_id)
+def _continue_task(task_ex_id):
+    from mistral.engine import task_handler
 
-    if not states.is_completed(task_ex.state):
-        msg = "Task timed out [id=%s, timeout(s)=%s]." % (task_ex_id, timeout)
+    with db_api.transaction():
+        task_handler.continue_task(db_api.get_task_execution(task_ex_id))
 
-        wf_trace.info(task_ex, msg)
 
-        wf_trace.info(
-            task_ex,
-            "Task '%s' [%s -> ERROR]" % (task_ex.name, task_ex.state)
+def _complete_task(task_ex_id, state, state_info):
+    from mistral.engine import task_handler
+
+    with db_api.transaction():
+        task_handler.complete_task(
+            db_api.get_task_execution(task_ex_id),
+            state,
+            state_info
         )
 
-        rpc.get_engine_client().on_task_state_change(
-            task_ex_id,
-            states.ERROR,
-            msg
-        )
+
+def _fail_task_if_incomplete(task_ex_id, timeout):
+    from mistral.engine import task_handler
+
+    with db_api.transaction():
+        task_ex = db_api.get_task_execution(task_ex_id)
+
+        if not states.is_completed(task_ex.state):
+            msg = 'Task timed out [timeout(s)=%s].' % timeout
+
+            task_handler.complete_task(
+                db_api.get_task_execution(task_ex_id),
+                states.ERROR,
+                msg
+            )
