@@ -13,12 +13,12 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import mock
-
 from oslo_config import cfg
 
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import models
+from mistral import exceptions as exc
+from mistral import expressions as expr
 from mistral.services import workflows as wf_service
 from mistral.tests.unit import base as test_base
 from mistral.tests.unit.engine import base as engine_test_base
@@ -43,14 +43,14 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
             task1:
               action: std.echo output="Hi"
               publish:
-                hi: <% $.task1 %>
+                hi: <% task(task1).result %>
               on-success:
                 - task2
 
             task2:
               action: std.echo output="Morpheus"
               publish:
-                to: <% $.task2 %>
+                to: <% task(task2).result %>
               on-success:
                 - task3
 
@@ -64,7 +64,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf', {}, env={'from': 'Neo'})
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         # Note: We need to reread execution to access related tasks.
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
@@ -85,6 +85,10 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
             task3.published
         )
 
+        # Make sure that task inbound context doesn't contain workflow
+        # execution info.
+        self.assertFalse('__execution' in task1.in_context)
+
     def test_linear_with_branches_dataflow(self):
         linear_with_branches_wf = """---
         version: '2.0'
@@ -96,7 +100,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
             task1:
               action: std.echo output="Hi"
               publish:
-                hi: <% $.task1 %>
+                hi: <% task(task1).result %>
                 progress: "completed task1"
               on-success:
                 - notify
@@ -105,7 +109,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
             task2:
               action: std.echo output="Morpheus"
               publish:
-                to: <% $.task2 %>
+                to: <% task(task2).result %>
                 progress: "completed task2"
               on-success:
                 - notify
@@ -121,7 +125,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
             notify:
               action: std.echo output=<% $.progress %>
               publish:
-                progress: <% $.notify %>
+                progress: <% task(notify).result %>
         """
 
         wf_service.create_workflows(linear_with_branches_wf)
@@ -129,7 +133,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf', {}, env={'from': 'Neo'})
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         # Note: We need to reread execution to access related tasks.
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
@@ -165,6 +169,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
         self.assertDictEqual(exp_published_arr[0], task1.published)
         self.assertDictEqual(exp_published_arr[1], task2.published)
         self.assertDictEqual(exp_published_arr[2], task3.published)
+
         self.assertIn(exp_published_arr[0]['progress'], notify_published_arr)
         self.assertIn(exp_published_arr[1]['progress'], notify_published_arr)
         self.assertIn(exp_published_arr[2]['progress'], notify_published_arr)
@@ -180,12 +185,12 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
             task1:
               action: std.echo output=1
               publish:
-                var1: <% $.task1 %>
+                var1: <% task(task1).result %>
 
             task2:
               action: std.echo output=2
               publish:
-                var2: <% $.task2 %>
+                var2: <% task(task2).result %>
         """
 
         wf_service.create_workflows(parallel_tasks_wf)
@@ -193,7 +198,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf', {})
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         # Note: We need to reread execution to access related tasks.
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
@@ -269,7 +274,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf', {})
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         # Note: We need to reread execution to access related tasks.
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
@@ -302,7 +307,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
 
         self.assertEqual(1, wf_ex.output['var1'])
         self.assertEqual(12, wf_ex.output['var12'])
-        self.assertFalse('var13' in wf_ex.output)
+        self.assertNotIn('var13', wf_ex.output)
         self.assertEqual(14, wf_ex.output['var14'])
         self.assertEqual(2, wf_ex.output['var2'])
         self.assertEqual(21, wf_ex.output['var21'])
@@ -318,21 +323,21 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
             task1:
               action: std.echo output="Hi"
               publish:
-                greeting: <% $.task1 %>
+                greeting: <% task(task1).result %>
               on-success:
                 - task2
 
             task2:
               action: std.echo output="Yo"
               publish:
-                greeting: <% $.task2 %>
+                greeting: <% task(task2).result %>
               on-success:
                 - task3
 
             task3:
               action: std.echo output="Morpheus"
               publish:
-                to: <% $.task3 %>
+                to: <% task(task3).result %>
               on-success:
                 - task4
 
@@ -350,7 +355,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
             env={'from': 'Neo'}
         )
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         # Note: We need to reread execution to access related tasks.
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
@@ -373,6 +378,58 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
             task4.published
         )
 
+    def test_sequential_tasks_publishing_same_structured(self):
+        var_overwrite_wf = """---
+        version: '2.0'
+
+        wf:
+          type: direct
+
+          tasks:
+            task1:
+              publish:
+                greeting: {"a": "b"}
+              on-success:
+                - task2
+
+            task2:
+              publish:
+                greeting: {}
+              on-success:
+                - task3
+
+            task3:
+              publish:
+                result: <% $.greeting %>
+        """
+
+        wf_service.create_workflows(var_overwrite_wf)
+
+        # Start workflow.
+        wf_ex = self.engine.start_workflow(
+            'wf',
+            {},
+            env={'from': 'Neo'}
+        )
+
+        self.await_workflow_success(wf_ex.id)
+
+        # Note: We need to reread execution to access related tasks.
+        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+        self.assertEqual(states.SUCCESS, wf_ex.state)
+
+        tasks = wf_ex.task_executions
+
+        task1 = self._assert_single_item(tasks, name='task1')
+        task2 = self._assert_single_item(tasks, name='task2')
+        task3 = self._assert_single_item(tasks, name='task3')
+
+        self.assertEqual(states.SUCCESS, task3.state)
+        self.assertDictEqual({'greeting': {'a': 'b'}}, task1.published)
+        self.assertDictEqual({'greeting': {}}, task2.published)
+        self.assertDictEqual({'result': {}}, task3.published)
+
     def test_linear_dataflow_implicit_publish(self):
         linear_wf = """---
         version: '2.0'
@@ -400,14 +457,16 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
             task4:
               join: all
               publish:
-                result: "<% $.task1 %>, <% $.task21 %>! Your <% $.task22 %>."
+                result: >
+                  <% task(task1).result %>, <% task(task21).result %>!
+                  Your <% task(task22).result %>.
         """
         wf_service.create_workflows(linear_wf)
 
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf', {})
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         # Note: We need to reread execution to access related tasks.
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
@@ -416,7 +475,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
         task4 = self._assert_single_item(tasks, name='task4')
 
         self.assertDictEqual(
-            {'result': 'Hi, Morpheus! Your Neo.'},
+            {'result': 'Hi, Morpheus! Your Neo.\n'},
             task4.published
         )
 
@@ -431,7 +490,7 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
             task1:
               action: std.echo output=["Hi", "John Doe!"]
               publish:
-                hi: <% $.task1 %>
+                hi: <% task(task1).result %>
               keep-result: false
 
         """
@@ -440,12 +499,13 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf', {})
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         # Note: We need to reread execution to access related tasks.
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         tasks = wf_ex.task_executions
+
         task1 = self._assert_single_item(tasks, name='task1')
 
         result = data_flow.get_task_execution_result(task1)
@@ -471,22 +531,22 @@ class DataFlowEngineTest(engine_test_base.EngineTestCase):
                with-items: i in <% list() %>
                action: std.echo output= "Task 1.<% $.i %>"
                publish:
-                 result: <% $.task1 %>
+                 result: <% task(task1).result %>
         """
         wf_service.create_workflows(wf)
 
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf1_with_items', {})
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         # Note: We need to reread execution to access related tasks.
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        tasks = wf_ex.task_executions
-        task1 = self._assert_single_item(tasks, name='task1')
+        task1 = self._assert_single_item(wf_ex.task_executions, name='task1')
 
         result = data_flow.get_task_execution_result(task1)
+
         self.assertListEqual([], result)
 
 
@@ -498,43 +558,141 @@ class DataFlowTest(test_base.BaseTest):
                 "version": '2.0',
                 'name': 'task1',
                 'with-items': 'var in [1]',
-                'type': 'direct'
+                'type': 'direct',
+                'action': 'my_action'
             },
             runtime_context={
                 'with_items_context': {'count': 1}
             }
         )
 
-        action_exs = []
-
-        action_exs.append(models.ActionExecution(
+        task_ex.action_executions = [models.ActionExecution(
             name='my_action',
             output={'result': 1},
             accepted=True,
-            runtime_context={'with_items_index': 0}
-        ))
+            runtime_context={'index': 0}
+        )]
 
-        with mock.patch.object(db_api, 'get_action_executions',
-                               return_value=action_exs):
-            self.assertEqual([1], data_flow.get_task_execution_result(task_ex))
+        self.assertEqual([1], data_flow.get_task_execution_result(task_ex))
 
-        action_exs.append(models.ActionExecution(
+        task_ex.action_executions.append(models.ActionExecution(
             name='my_action',
             output={'result': 1},
             accepted=True,
-            runtime_context={'with_items_index': 0}
+            runtime_context={'index': 0}
         ))
 
-        action_exs.append(models.ActionExecution(
+        task_ex.action_executions.append(models.ActionExecution(
             name='my_action',
             output={'result': 1},
             accepted=False,
-            runtime_context={'with_items_index': 0}
+            runtime_context={'index': 0}
         ))
 
-        with mock.patch.object(db_api, 'get_action_executions',
-                               return_value=action_exs):
-            self.assertEqual(
-                [1, 1],
-                data_flow.get_task_execution_result(task_ex)
-            )
+        self.assertEqual(
+            [1, 1],
+            data_flow.get_task_execution_result(task_ex)
+        )
+
+    def test_context_view(self):
+        ctx = data_flow.ContextView(
+            {
+                'k1': 'v1',
+                'k11': 'v11',
+                'k3': 'v3'
+            },
+            {
+                'k2': 'v2',
+                'k21': 'v21',
+                'k3': 'v32'
+            }
+        )
+
+        self.assertIsInstance(ctx, dict)
+        self.assertEqual(5, len(ctx))
+
+        self.assertIn('k1', ctx)
+        self.assertIn('k11', ctx)
+        self.assertIn('k3', ctx)
+        self.assertIn('k2', ctx)
+        self.assertIn('k21', ctx)
+
+        self.assertEqual('v1', ctx['k1'])
+        self.assertEqual('v1', ctx.get('k1'))
+        self.assertEqual('v11', ctx['k11'])
+        self.assertEqual('v11', ctx.get('k11'))
+        self.assertEqual('v3', ctx['k3'])
+        self.assertEqual('v2', ctx['k2'])
+        self.assertEqual('v2', ctx.get('k2'))
+        self.assertEqual('v21', ctx['k21'])
+        self.assertEqual('v21', ctx.get('k21'))
+
+        self.assertIsNone(ctx.get('Not existing key'))
+
+        self.assertRaises(exc.MistralError, ctx.update)
+        self.assertRaises(exc.MistralError, ctx.clear)
+        self.assertRaises(exc.MistralError, ctx.pop, 'k1')
+        self.assertRaises(exc.MistralError, ctx.popitem)
+        self.assertRaises(exc.MistralError, ctx.__setitem__, 'k5', 'v5')
+        self.assertRaises(exc.MistralError, ctx.__delitem__, 'k2')
+
+        self.assertEqual('v1', expr.evaluate('<% $.k1 %>', ctx))
+        self.assertEqual('v2', expr.evaluate('<% $.k2 %>', ctx))
+        self.assertEqual('v3', expr.evaluate('<% $.k3 %>', ctx))
+
+        # Now change the order of dictionaries and make sure to have
+        # a different for key 'k3'.
+        ctx = data_flow.ContextView(
+            {
+                'k2': 'v2',
+                'k21': 'v21',
+                'k3': 'v32'
+            },
+            {
+                'k1': 'v1',
+                'k11': 'v11',
+                'k3': 'v3'
+            }
+        )
+
+        self.assertEqual('v32', expr.evaluate('<% $.k3 %>', ctx))
+
+    def test_context_view_eval_root_with_yaql(self):
+        ctx = data_flow.ContextView(
+            {'k1': 'v1'},
+            {'k2': 'v2'}
+        )
+
+        res = expr.evaluate('<% $ %>', ctx)
+
+        self.assertIsNotNone(res)
+        self.assertIsInstance(res, dict)
+        self.assertEqual(2, len(res))
+
+    def test_context_view_eval_keys(self):
+        ctx = data_flow.ContextView(
+            {'k1': 'v1'},
+            {'k2': 'v2'}
+        )
+
+        res = expr.evaluate('<% $.keys() %>', ctx)
+
+        self.assertIsNotNone(res)
+        self.assertIsInstance(res, list)
+        self.assertEqual(2, len(res))
+        self.assertIn('k1', res)
+        self.assertIn('k2', res)
+
+    def test_context_view_eval_values(self):
+        ctx = data_flow.ContextView(
+            {'k1': 'v1'},
+            {'k2': 'v2'}
+        )
+
+        res = expr.evaluate('<% $.values() %>', ctx)
+
+        self.assertIsNotNone(res)
+        self.assertIsInstance(res, list)
+        self.assertEqual(2, len(res))
+        self.assertIn('v1', res)
+        self.assertIn('v2', res)
