@@ -15,6 +15,7 @@
 import mock
 
 from oslo_config import cfg
+import testtools
 
 from mistral.actions import std_actions
 from mistral.db.v2 import api as db_api
@@ -31,10 +32,13 @@ cfg.CONF.set_default('auth_enable', False, group='pecan')
 SIMPLE_WORKBOOK = """
 ---
 version: '2.0'
+
 name: wb1
+
 workflows:
   wf1:
     type: direct
+
     tasks:
       t1:
         action: std.echo output="Task 1"
@@ -51,10 +55,13 @@ workflows:
 SIMPLE_WORKBOOK_DIFF_ENV_VAR = """
 ---
 version: '2.0'
+
 name: wb1
+
 workflows:
   wf1:
     type: direct
+
     tasks:
       t10:
         action: std.echo output="Task 10"
@@ -78,16 +85,19 @@ workflows:
 WITH_ITEMS_WORKBOOK = """
 ---
 version: '2.0'
+
 name: wb3
+
 workflows:
   wf1:
     type: direct
+
     tasks:
       t1:
         with-items: i in <% list(range(0, 3)) %>
         action: std.echo output="Task 1.<% $.i %>"
         publish:
-          v1: <% $.t1 %>
+          v1: <% task(t1).result %>
         on-success:
           - t2
       t2:
@@ -97,16 +107,19 @@ workflows:
 WITH_ITEMS_WORKBOOK_DIFF_ENV_VAR = """
 ---
 version: '2.0'
+
 name: wb3
+
 workflows:
   wf1:
     type: direct
+
     tasks:
       t1:
         with-items: i in <% list(range(0, 3)) %>
         action: std.echo output="Task 1.<% $.i %> [<% env().var1 %>]"
         publish:
-          v1: <% $.t1 %>
+          v1: <% task(t1).result %>
         on-success:
           - t2
       t2:
@@ -116,17 +129,20 @@ workflows:
 WITH_ITEMS_WORKBOOK_CONCURRENCY = """
 ---
 version: '2.0'
+
 name: wb3
+
 workflows:
   wf1:
     type: direct
+
     tasks:
       t1:
         with-items: i in <% list(range(0, 4)) %>
         action: std.echo output="Task 1.<% $.i %>"
         concurrency: 2
         publish:
-          v1: <% $.t1 %>
+          v1: <% task(t1).result %>
         on-success:
           - t2
       t2:
@@ -136,10 +152,13 @@ workflows:
 JOIN_WORKBOOK = """
 ---
 version: '2.0'
+
 name: wb1
+
 workflows:
   wf1:
     type: direct
+
     tasks:
       t1:
         action: std.echo output="Task 1"
@@ -156,10 +175,13 @@ workflows:
 
 SUBFLOW_WORKBOOK = """
 version: '2.0'
+
 name: wb1
+
 workflows:
   wf1:
     type: direct
+
     tasks:
       t1:
         action: std.echo output="Task 1"
@@ -171,10 +193,13 @@ workflows:
           - t3
       t3:
         action: std.echo output="Task 3"
+
   wf2:
     type: direct
+
     output:
-      result: <% $.wf2_t1 %>
+      result: <% task(wf2_t1).result %>
+
     tasks:
       wf2_t1:
         action: std.echo output="Task 2"
@@ -200,7 +225,9 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb1.wf1', {})
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+
+        self.await_workflow_error(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -215,14 +242,16 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertIsNotNone(task_2_ex.state_info)
 
         # Resume workflow and re-run failed task.
-        self.engine.rerun_workflow(wf_ex.id, task_2_ex.id)
+        self.engine.rerun_workflow(task_2_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
         # Wait for the workflow to succeed.
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
@@ -290,24 +319,27 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb1.wf1', {}, env=env)
-        self._await(lambda: self.is_execution_error(wf_ex.id))
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+        self.await_workflow_error(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.ERROR, wf_ex.state)
         self.assertIsNotNone(wf_ex.state_info)
-        self.assertEqual(3, len(wf_ex.task_executions))
+        self.assertEqual(3, len(task_execs))
         self.assertDictEqual(env, wf_ex.params['env'])
         self.assertDictEqual(env, wf_ex.context['__env'])
 
-        task_exs = wf_ex.task_executions
-        task_10_ex = self._assert_single_item(task_exs, name='t10')
-        task_21_ex = self._assert_single_item(task_exs, name='t21')
-        task_30_ex = self._assert_single_item(task_exs, name='t30')
+        task_10_ex = self._assert_single_item(task_execs, name='t10')
+        task_21_ex = self._assert_single_item(task_execs, name='t21')
+        task_30_ex = self._assert_single_item(task_execs, name='t30')
 
         self.assertEqual(states.SUCCESS, task_10_ex.state)
         self.assertEqual(states.ERROR, task_21_ex.state)
         self.assertIsNotNone(task_21_ex.state_info)
-        self.assertEqual(states.WAITING, task_30_ex.state)
+        self.assertEqual(states.ERROR, task_30_ex.state)
 
         # Update env in workflow execution with the following.
         updated_env = {
@@ -317,27 +349,31 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         }
 
         # Resume workflow and re-run failed task.
-        self.engine.rerun_workflow(wf_ex.id, task_21_ex.id, env=updated_env)
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        wf_ex = self.engine.rerun_workflow(task_21_ex.id, env=updated_env)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
         self.assertDictEqual(updated_env, wf_ex.params['env'])
         self.assertDictEqual(updated_env, wf_ex.context['__env'])
 
+        # Await t30 success.
+        self.await_task_success(task_30_ex.id)
+
         # Wait for the workflow to succeed.
-        self._await(lambda: self.is_execution_success(wf_ex.id))
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        self.await_workflow_success(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
-        self.assertEqual(4, len(wf_ex.task_executions))
+        self.assertEqual(4, len(task_execs))
 
-        task_exs = wf_ex.task_executions
-        task_10_ex = self._assert_single_item(task_exs, name='t10')
-        task_21_ex = self._assert_single_item(task_exs, name='t21')
-        task_22_ex = self._assert_single_item(task_exs, name='t22')
-        task_30_ex = self._assert_single_item(task_exs, name='t30')
+        task_10_ex = self._assert_single_item(task_execs, name='t10')
+        task_21_ex = self._assert_single_item(task_execs, name='t21')
+        task_22_ex = self._assert_single_item(task_execs, name='t22')
+        task_30_ex = self._assert_single_item(task_execs, name='t30')
 
         # Check action executions of task 10.
         self.assertEqual(states.SUCCESS, task_10_ex.state)
@@ -421,25 +457,35 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb1.wf1', {})
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+
+        self.await_workflow_error(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
         self.assertIsNotNone(wf_ex.state_info)
-        self.assertEqual(2, len(wf_ex.task_executions))
 
-        task_1_ex = self._assert_single_item(wf_ex.task_executions, name='t1')
-        task_2_ex = self._assert_single_item(wf_ex.task_executions, name='t2')
+        task_execs = wf_ex.task_executions
+
+        self.assertEqual(2, len(task_execs))
+
+        task_1_ex = self._assert_single_item(
+            task_execs,
+            name='t1',
+            state=states.SUCCESS
+        )
+        task_2_ex = self._assert_single_item(
+            task_execs,
+            name='t2',
+            state=states.ERROR
+        )
 
-        self.assertEqual(states.SUCCESS, task_1_ex.state)
-        self.assertEqual(states.ERROR, task_2_ex.state)
         self.assertIsNotNone(task_2_ex.state_info)
 
         # Resume workflow and re-run failed task.
         e = self.assertRaises(
-            exc.EngineException,
+            exc.MistralError,
             self.engine.rerun_workflow,
-            wf_ex.id,
             task_1_ex.id
         )
 
@@ -464,14 +510,19 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb3.wf1', {})
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+
+        self.await_workflow_error(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
         self.assertIsNotNone(wf_ex.state_info)
-        self.assertEqual(1, len(wf_ex.task_executions))
 
-        task_1_ex = self._assert_single_item(wf_ex.task_executions, name='t1')
+        task_execs = wf_ex.task_executions
+
+        self.assertEqual(1, len(task_execs))
+
+        task_1_ex = self._assert_single_item(task_execs, name='t1')
 
         self.assertEqual(states.ERROR, task_1_ex.state)
         self.assertIsNotNone(task_1_ex.state_info)
@@ -483,21 +534,26 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(3, len(task_1_action_exs))
 
         # Resume workflow and re-run failed task.
-        self.engine.rerun_workflow(wf_ex.id, task_1_ex.id, reset=False)
+        self.engine.rerun_workflow(task_1_ex.id, reset=False)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
-        self._await(lambda: self.is_execution_success(wf_ex.id), delay=10)
+        self.await_workflow_success(wf_ex.id, delay=10)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
-        self.assertEqual(2, len(wf_ex.task_executions))
 
-        task_1_ex = self._assert_single_item(wf_ex.task_executions, name='t1')
-        task_2_ex = self._assert_single_item(wf_ex.task_executions, name='t2')
+        task_execs = wf_ex.task_executions
+
+        self.assertEqual(2, len(task_execs))
+
+        task_1_ex = self._assert_single_item(task_execs, name='t1')
+        task_2_ex = self._assert_single_item(task_execs, name='t2')
 
         # Check action executions of task 1.
         self.assertEqual(states.SUCCESS, task_1_ex.state)
@@ -509,9 +565,10 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # The single action execution that succeeded should not re-run.
         self.assertEqual(5, len(task_1_action_exs))
-
-        self.assertListEqual(['Task 1.0', 'Task 1.1', 'Task 1.2'],
-                             task_1_ex.published.get('v1'))
+        self.assertListEqual(
+            ['Task 1.0', 'Task 1.1', 'Task 1.2'],
+            task_1_ex.published.get('v1')
+        )
 
         # Check action executions of task 2.
         self.assertEqual(states.SUCCESS, task_2_ex.state)
@@ -522,6 +579,7 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         self.assertEqual(1, len(task_2_action_exs))
 
+    @testtools.skip('Restore concurrency support.')
     @mock.patch.object(
         std_actions.EchoAction,
         'run',
@@ -542,7 +600,9 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb3.wf1', {})
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+
+        self.await_workflow_error(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -560,13 +620,15 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(4, len(task_1_action_exs))
 
         # Resume workflow and re-run failed task.
-        self.engine.rerun_workflow(wf_ex.id, task_1_ex.id, reset=False)
+        self.engine.rerun_workflow(task_1_ex.id, reset=False)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
-        self._await(lambda: self.is_execution_success(wf_ex.id), delay=10)
+        self.await_workflow_success(wf_ex.id, delay=10)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
@@ -616,13 +678,13 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         wb_service.create_workbook_v2(WITH_ITEMS_WORKBOOK_DIFF_ENV_VAR)
 
         # Initial environment variables for the workflow execution.
-        env = {
-            'var1': 'fee fi fo fum'
-        }
+        env = {'var1': 'fee fi fo fum'}
 
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb3.wf1', {}, env=env)
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+
+        self.await_workflow_error(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -647,7 +709,6 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # Resume workflow and re-run failed task.
         self.engine.rerun_workflow(
-            wf_ex.id,
             task_1_ex.id,
             reset=False,
             env=updated_env
@@ -658,7 +719,8 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
-        self._await(lambda: self.is_execution_success(wf_ex.id), delay=10)
+        self.await_workflow_success(wf_ex.id, delay=10)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
@@ -718,8 +780,11 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb1.wf1', {})
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+
+        self.await_workflow_error(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -736,14 +801,16 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertIsNotNone(task_3_ex.state_info)
 
         # Resume workflow and re-run failed task.
-        self.engine.rerun_workflow(wf_ex.id, task_3_ex.id)
+        self.engine.rerun_workflow(task_3_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
         # Wait for the workflow to succeed.
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
@@ -804,70 +871,93 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb1.wf1', {})
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
-        task_1_ex = self._assert_single_item(wf_ex.task_executions, name='t1')
-        task_2_ex = self._assert_single_item(wf_ex.task_executions, name='t2')
-        self._await(lambda: self.is_task_in_state(task_1_ex.id, states.ERROR))
-        self._await(lambda: self.is_task_in_state(task_2_ex.id, states.ERROR))
 
-        self._await(lambda: self.is_execution_error(wf_ex.id))
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_execs = wf_ex.task_executions
+
+        task_1_ex = self._assert_single_item(task_execs, name='t1')
+        task_2_ex = self._assert_single_item(task_execs, name='t2')
+
+        self.await_task_error(task_1_ex.id)
+        self.await_task_error(task_2_ex.id)
+
+        self.await_workflow_error(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.ERROR, wf_ex.state)
         self.assertIsNotNone(wf_ex.state_info)
-        self.assertEqual(2, len(wf_ex.task_executions))
+        self.assertEqual(2, len(task_execs))
 
-        task_1_ex = self._assert_single_item(wf_ex.task_executions, name='t1')
-        task_2_ex = self._assert_single_item(wf_ex.task_executions, name='t2')
+        task_1_ex = self._assert_single_item(task_execs, name='t1')
 
         self.assertEqual(states.ERROR, task_1_ex.state)
         self.assertIsNotNone(task_1_ex.state_info)
+
+        task_2_ex = self._assert_single_item(task_execs, name='t2')
+
         self.assertEqual(states.ERROR, task_2_ex.state)
         self.assertIsNotNone(task_2_ex.state_info)
 
         # Resume workflow and re-run failed task.
-        self.engine.rerun_workflow(wf_ex.id, task_1_ex.id)
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        wf_ex = self.engine.rerun_workflow(task_1_ex.id)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_execs = wf_ex.task_executions
+
         # Wait for the task to succeed.
-        task_1_ex = self._assert_single_item(wf_ex.task_executions, name='t1')
-        self._await(lambda: self.is_task_in_state(task_1_ex.id,
-                                                  states.SUCCESS))
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        task_1_ex = self._assert_single_item(task_execs, name='t1')
+
+        self.await_task_success(task_1_ex.id)
+
+        self.await_workflow_error(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.ERROR, wf_ex.state)
         self.assertIsNotNone(wf_ex.state_info)
-        self.assertEqual(3, len(wf_ex.task_executions))
+        self.assertEqual(3, len(task_execs))
 
-        task_1_ex = self._assert_single_item(wf_ex.task_executions, name='t1')
-        task_2_ex = self._assert_single_item(wf_ex.task_executions, name='t2')
-        task_3_ex = self._assert_single_item(wf_ex.task_executions, name='t3')
+        task_1_ex = self._assert_single_item(task_execs, name='t1')
+        task_2_ex = self._assert_single_item(task_execs, name='t2')
+        task_3_ex = self._assert_single_item(task_execs, name='t3')
 
         self.assertEqual(states.SUCCESS, task_1_ex.state)
         self.assertEqual(states.ERROR, task_2_ex.state)
-        self.assertEqual(states.WAITING, task_3_ex.state)
+        self.assertEqual(states.ERROR, task_3_ex.state)
 
         # Resume workflow and re-run failed task.
-        self.engine.rerun_workflow(wf_ex.id, task_2_ex.id)
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        wf_ex = self.engine.rerun_workflow(task_2_ex.id)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
+        # Join now should finally complete.
+        self.await_task_success(task_3_ex.id)
+
         # Wait for the workflow to succeed.
-        self._await(lambda: self.is_execution_success(wf_ex.id))
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        self.await_workflow_success(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
-        self.assertEqual(3, len(wf_ex.task_executions))
+        self.assertEqual(3, len(task_execs))
 
-        task_1_ex = self._assert_single_item(wf_ex.task_executions, name='t1')
-        task_2_ex = self._assert_single_item(wf_ex.task_executions, name='t2')
-        task_3_ex = self._assert_single_item(wf_ex.task_executions, name='t3')
+        task_1_ex = self._assert_single_item(task_execs, name='t1')
+        task_2_ex = self._assert_single_item(task_execs, name='t2')
+        task_3_ex = self._assert_single_item(task_execs, name='t3')
 
         # Check action executions of task 1.
         self.assertEqual(states.SUCCESS, task_1_ex.state)
@@ -897,7 +987,7 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(states.SUCCESS, task_3_ex.state)
 
         task_3_action_exs = db_api.get_action_executions(
-            task_execution_id=wf_ex.task_executions[2].id
+            task_execution_id=task_execs[2].id
         )
 
         self.assertEqual(1, len(task_3_action_exs))
@@ -929,7 +1019,9 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb3.wf1', {})
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+
+        self.await_workflow_error(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -938,7 +1030,8 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         task_1_ex = self._assert_single_item(wf_ex.task_executions, name='t1')
 
-        self.assertEqual(states.ERROR, task_1_ex.state)
+        self.await_task_error(task_1_ex.id)
+
         self.assertIsNotNone(task_1_ex.state_info)
 
         task_1_action_exs = db_api.get_action_executions(
@@ -948,13 +1041,13 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(3, len(task_1_action_exs))
 
         # Resume workflow and re-run failed task. Re-run #1 with no reset.
-        self.engine.rerun_workflow(wf_ex.id, task_1_ex.id, reset=False)
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        wf_ex = self.engine.rerun_workflow(task_1_ex.id, reset=False)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
-        self._await(lambda: self.is_execution_error(wf_ex.id), delay=10)
+        self.await_workflow_error(wf_ex.id, delay=10)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -967,13 +1060,15 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(5, len(task_1_action_exs))
 
         # Resume workflow and re-run failed task. Re-run #2 with reset.
-        self.engine.rerun_workflow(wf_ex.id, task_1_ex.id, reset=True)
+        self.engine.rerun_workflow(task_1_ex.id, reset=True)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
-        self._await(lambda: self.is_execution_error(wf_ex.id), delay=10)
+        self.await_workflow_error(wf_ex.id, delay=10)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -986,13 +1081,15 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(8, len(task_1_action_exs))
 
         # Resume workflow and re-run failed task. Re-run #3 with no reset.
-        self.engine.rerun_workflow(wf_ex.id, task_1_ex.id, reset=False)
+        self.engine.rerun_workflow(task_1_ex.id, reset=False)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
-        self._await(lambda: self.is_execution_error(wf_ex.id), delay=10)
+        self.await_workflow_error(wf_ex.id, delay=10)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -1005,13 +1102,15 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(10, len(task_1_action_exs))
 
         # Resume workflow and re-run failed task. Re-run #4 with no reset.
-        self.engine.rerun_workflow(wf_ex.id, task_1_ex.id, reset=False)
+        self.engine.rerun_workflow(task_1_ex.id, reset=False)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
-        self._await(lambda: self.is_execution_success(wf_ex.id), delay=10)
+        self.await_workflow_success(wf_ex.id, delay=10)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
@@ -1061,7 +1160,9 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb1.wf1', {})
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+
+        self.await_workflow_error(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -1076,14 +1177,15 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertIsNotNone(task_2_ex.state_info)
 
         # Resume workflow and re-run failed task.
-        self.engine.rerun_workflow(wf_ex.id, task_2_ex.id)
+        self.engine.rerun_workflow(task_2_ex.id)
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.RUNNING, wf_ex.state)
         self.assertIsNone(wf_ex.state_info)
 
         # Wait for the workflow to succeed.
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
@@ -1107,8 +1209,9 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(states.SUCCESS, task_2_ex.state)
         self.assertIsNone(task_2_ex.state_info)
 
-        task_2_action_exs = db_api.get_action_executions(
-            task_execution_id=task_2_ex.id)
+        task_2_action_exs = db_api.get_workflow_executions(
+            task_execution_id=task_2_ex.id
+        )
 
         self.assertEqual(2, len(task_2_action_exs))
         self.assertEqual(states.ERROR, task_2_action_exs[0].state)
@@ -1140,7 +1243,9 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
 
         # Run workflow and fail task.
         wf_ex = self.engine.start_workflow('wb1.wf1', {})
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+
+        self.await_workflow_error(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.ERROR, wf_ex.state)
@@ -1174,7 +1279,8 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertIsNotNone(sub_wf_task_ex.state_info)
 
         # Resume workflow and re-run failed subworkflow task.
-        self.engine.rerun_workflow(sub_wf_ex.id, sub_wf_task_ex.id)
+        self.engine.rerun_workflow(sub_wf_task_ex.id)
+
         sub_wf_ex = db_api.get_workflow_execution(sub_wf_ex.id)
 
         self.assertEqual(states.RUNNING, sub_wf_ex.state)
@@ -1186,7 +1292,8 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertIsNone(wf_ex.state_info)
 
         # Wait for the subworkflow to succeed.
-        self._await(lambda: self.is_execution_success(sub_wf_ex.id))
+        self.await_workflow_success(sub_wf_ex.id)
+
         sub_wf_ex = db_api.get_workflow_execution(sub_wf_ex.id)
 
         self.assertEqual(states.SUCCESS, sub_wf_ex.state)
@@ -1210,7 +1317,8 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(states.SUCCESS, sub_wf_task_ex_action_exs[1].state)
 
         # Wait for the main workflow to succeed.
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
@@ -1234,8 +1342,9 @@ class DirectWorkflowRerunTest(base.EngineTestCase):
         self.assertEqual(states.SUCCESS, task_2_ex.state)
         self.assertIsNone(task_2_ex.state_info)
 
-        task_2_action_exs = db_api.get_action_executions(
-            task_execution_id=task_2_ex.id)
+        task_2_action_exs = db_api.get_workflow_executions(
+            task_execution_id=task_2_ex.id
+        )
 
         self.assertEqual(1, len(task_2_action_exs))
         self.assertEqual(states.SUCCESS, task_1_action_exs[0].state)
