@@ -41,14 +41,14 @@ wf:
     task1:
       action: std.echo output=<% $.num1 %>
       publish:
-        result1: <% $.task1 %>
+        result1: <% task(task1).result %>
       on-complete:
         - task3
 
     task2:
       action: std.echo output=<% $.num2 %>
       publish:
-        result2: <% $.task2 %>
+        result2: <% task(task2).result %>
       on-complete:
         - task3
 
@@ -64,12 +64,12 @@ wf:
     task4:
       action: std.echo output=4
       publish:
-        result: <% $.task4 %>
+        result: <% task(task4).result %>
 
     task5:
       action: std.echo output=5
       publish:
-        result: <% $.task5 %>
+        result: <% task(task5).result %>
 """
 
 
@@ -80,7 +80,7 @@ class NoopTaskEngineTest(base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf', {'num1': 1, 'num2': 1})
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         # Note: We need to reread execution to access related tasks.
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
@@ -107,7 +107,7 @@ class NoopTaskEngineTest(base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf', {'num1': 1, 'num2': 2})
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         # Note: We need to reread execution to access related tasks.
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
