@@ -17,6 +17,7 @@ from oslo_config import cfg
 
 from mistral.actions import std_actions
 from mistral.db.v2 import api as db_api
+from mistral.db.v2.sqlalchemy import models
 from mistral.engine import policies
 from mistral import exceptions as exc
 from mistral.services import workbooks as wb_service
@@ -364,18 +365,29 @@ class PoliciesTest(base.EngineTestCase):
                 "delay": {"type": "integer"}
             }
         }
-        task_db = type('Task', (object,), {'in_context': {'int_var': 5}})
+
+        wf_ex = models.WorkflowExecution(
+            id='1-2-3-4',
+            context={},
+            input={}
+        )
+
+        task_ex = models.TaskExecution(in_context={'int_var': 5})
+        task_ex.workflow_execution = wf_ex
+
         policy.delay = "<% $.int_var %>"
 
         # Validation is ok.
-        policy.before_task_start(task_db, None)
+        policy.before_task_start(task_ex, None)
 
         policy.delay = "some_string"
 
         # Validation is failing now.
         exception = self.assertRaises(
             exc.InvalidModelException,
-            policy.before_task_start, task_db, None
+            policy.before_task_start,
+            task_ex,
+            None
         )
 
         self.assertIn("Invalid data type in TaskPolicy", str(exception))
@@ -422,7 +434,7 @@ class PoliciesTest(base.EngineTestCase):
             task_ex.runtime_context
         )
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
     def test_wait_before_policy_from_var(self):
         wb_service.create_workbook_v2(WAIT_BEFORE_FROM_VAR)
@@ -431,12 +443,40 @@ class PoliciesTest(base.EngineTestCase):
         exec_db = self.engine.start_workflow('wb.wf1', {'wait_before': 1})
 
         # Note: We need to reread execution to access related tasks.
-        exec_db = db_api.get_execution(exec_db.id)
+        exec_db = db_api.get_workflow_execution(exec_db.id)
         task_db = exec_db.task_executions[0]
 
         self.assertEqual(states.RUNNING_DELAYED, task_db.state)
 
-        self._await(lambda: self.is_execution_success(exec_db.id))
+        self.await_workflow_success(exec_db.id)
+
+    def test_wait_before_policy_two_tasks(self):
+        wf_text = """---
+        version: '2.0'
+
+        wf:
+          tasks:
+            a:
+              wait-before: 2
+              on-success: b
+            b:
+              action: std.noop
+
+        """
+
+        wf_service.create_workflows(wf_text)
+
+        wf_ex = self.engine.start_workflow('wf', {})
+
+        self.await_workflow_success(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+            task_execs = wf_ex.task_executions
+
+        self.assertEqual(2, len(task_execs))
+
+        self._assert_multiple_items(task_execs, 2, state=states.SUCCESS)
 
     def test_wait_after_policy(self):
         wb_service.create_workbook_v2(WAIT_AFTER_WB)
@@ -451,11 +491,8 @@ class PoliciesTest(base.EngineTestCase):
         self.assertEqual(states.RUNNING, task_ex.state)
         self.assertDictEqual({}, task_ex.runtime_context)
 
-        self._await(
-            lambda: self.is_task_delayed(task_ex.id),
-            delay=0.5
-        )
-        self._await(lambda: self.is_task_success(task_ex.id))
+        self.await_task_delayed(task_ex.id, delay=0.5)
+        self.await_task_success(task_ex.id)
 
     def test_wait_after_policy_from_var(self):
         wb_service.create_workbook_v2(WAIT_AFTER_FROM_VAR)
@@ -469,7 +506,11 @@ class PoliciesTest(base.EngineTestCase):
 
         self.assertEqual(states.RUNNING, task_ex.state)
         self.assertDictEqual({}, task_ex.runtime_context)
-        self.assertEqual(2, task_ex.in_context['wait_after'])
+
+        # TODO(rakhmerov): This check doesn't make sense anymore because
+        # we don't store evaluated value anywhere.
+        # Need to create a better test.
+        # self.assertEqual(2, task_ex.in_context['wait_after'])
 
     def test_retry_policy(self):
         wb_service.create_workbook_v2(RETRY_WB)
@@ -484,13 +525,10 @@ class PoliciesTest(base.EngineTestCase):
         self.assertEqual(states.RUNNING, task_ex.state)
         self.assertDictEqual({}, task_ex.runtime_context)
 
-        self._await(
-            lambda: self.is_task_delayed(task_ex.id),
-            delay=0.5
-        )
-        self._await(lambda: self.is_task_error(task_ex.id))
+        self.await_task_delayed(task_ex.id, delay=0.5)
+        self.await_task_error(task_ex.id)
 
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+        self.await_workflow_error(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
@@ -513,8 +551,11 @@ class PoliciesTest(base.EngineTestCase):
         self.assertEqual(states.RUNNING, task_ex.state)
         self.assertDictEqual({}, task_ex.runtime_context)
 
-        self.assertEqual(3, task_ex.in_context["count"])
-        self.assertEqual(1, task_ex.in_context["delay"])
+        # TODO(rakhmerov): This check doesn't make sense anymore because
+        # we don't store evaluated values anywhere.
+        # Need to create a better test.
+        # self.assertEqual(3, task_ex.in_context["count"])
+        # self.assertEqual(1, task_ex.in_context["delay"])
 
     def test_retry_policy_never_happen(self):
         retry_wb = """---
@@ -540,9 +581,9 @@ class PoliciesTest(base.EngineTestCase):
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
 
-        self._await(lambda: self.is_task_success(task_ex.id))
+        self.await_task_success(task_ex.id)
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
@@ -579,9 +620,9 @@ class PoliciesTest(base.EngineTestCase):
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
 
-        self._await(lambda: self.is_task_error(task_ex.id))
+        self.await_task_error(task_ex.id)
 
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+        self.await_workflow_error(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
@@ -618,9 +659,9 @@ class PoliciesTest(base.EngineTestCase):
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
 
-        self._await(lambda: self.is_task_error(task_ex.id))
+        self.await_task_error(task_ex.id)
 
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+        self.await_workflow_error(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
@@ -647,7 +688,7 @@ class PoliciesTest(base.EngineTestCase):
                 retry:
                   count: 4
                   delay: 1
-                  continue-on: <% $.task1 < 3 %>
+                  continue-on: <% task(task1).result < 3 %>
         """
         wb_service.create_workbook_v2(retry_wb)
 
@@ -658,9 +699,9 @@ class PoliciesTest(base.EngineTestCase):
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
 
-        self._await(lambda: self.is_task_success(task_ex.id))
+        self.await_task_success(task_ex.id)
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
@@ -684,7 +725,7 @@ class PoliciesTest(base.EngineTestCase):
                 retry:
                   count: 4
                   delay: 1
-                  continue-on: <% $.task1 <= 3 %>
+                  continue-on: <% task(task1).result <= 3 %>
         """
         wb_service.create_workbook_v2(retry_wb)
 
@@ -695,9 +736,9 @@ class PoliciesTest(base.EngineTestCase):
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
 
-        self._await(lambda: self.is_task_success(task_ex.id))
+        self.await_task_success(task_ex.id)
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
@@ -731,9 +772,9 @@ class PoliciesTest(base.EngineTestCase):
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
 
-        self._await(lambda: self.is_task_error(task_ex.id))
+        self.await_task_error(task_ex.id)
 
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+        self.await_workflow_error(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
@@ -774,8 +815,8 @@ class PoliciesTest(base.EngineTestCase):
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
 
-        self._await(lambda: self.is_task_error(task_ex.id))
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+        self.await_task_error(task_ex.id)
+        self.await_workflow_error(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
@@ -799,7 +840,8 @@ class PoliciesTest(base.EngineTestCase):
         workflows:
           wf1:
             output:
-              result: <% $.task1 %>
+              result: <% task(task1).result %>
+
             tasks:
               task1:
                 action: std.echo output="mocked result"
@@ -816,9 +858,9 @@ class PoliciesTest(base.EngineTestCase):
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
 
-        self._await(lambda: self.is_task_success(task_ex.id))
+        self.await_task_success(task_ex.id)
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = wf_ex.task_executions[0]
@@ -842,15 +884,13 @@ class PoliciesTest(base.EngineTestCase):
 
         self.assertEqual(states.RUNNING, task_ex.state)
 
-        self._await(lambda: self.is_task_error(task_ex.id))
+        self.await_task_error(task_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
-        task_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task1'
-        )
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self._assert_single_item(wf_ex.task_executions, name='task1')
+
+        self.await_workflow_success(wf_ex.id)
 
     def test_timeout_policy_success_after_timeout(self):
         wb_service.create_workbook_v2(TIMEOUT_WB2)
@@ -864,7 +904,7 @@ class PoliciesTest(base.EngineTestCase):
 
         self.assertEqual(states.RUNNING, task_ex.state)
 
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+        self.await_workflow_error(wf_ex.id)
 
         # Wait until timeout exceeds.
         self._sleep(1)
@@ -887,7 +927,10 @@ class PoliciesTest(base.EngineTestCase):
 
         self.assertEqual(states.RUNNING, task_ex.state)
 
-        self.assertEqual(1, task_ex.in_context['timeout'])
+        # TODO(rakhmerov): This check doesn't make sense anymore because
+        # we don't store evaluated 'timeout' value anywhere.
+        # Need to create a better test.
+        # self.assertEqual(1, task_ex.in_context['timeout'])
 
     def test_pause_before_policy(self):
         wb_service.create_workbook_v2(PAUSE_BEFORE_WB)
@@ -903,14 +946,16 @@ class PoliciesTest(base.EngineTestCase):
 
         self.assertEqual(states.IDLE, task_ex.state)
 
-        self._await(lambda: self.is_execution_paused(wf_ex.id))
+        self.await_workflow_paused(wf_ex.id)
+
         self._sleep(1)
+
         self.engine.resume_workflow(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         self._assert_single_item(wf_ex.task_executions, name='task1')
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = self._assert_single_item(
@@ -940,7 +985,7 @@ class PoliciesTest(base.EngineTestCase):
         self.assertEqual(states.IDLE, task_ex.state)
 
         # Verify wf paused by pause-before
-        self._await(lambda: self.is_execution_paused(wf_ex.id))
+        self.await_workflow_paused(wf_ex.id)
 
         # Allow wait-before to expire
         self._sleep(2)
@@ -948,7 +993,7 @@ class PoliciesTest(base.EngineTestCase):
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         # Verify wf still paused (wait-before didn't reactivate)
-        self._await(lambda: self.is_execution_paused(wf_ex.id))
+        self.await_workflow_paused(wf_ex.id)
 
         task_ex = db_api.get_task_execution(task_ex.id)
         self.assertEqual(states.IDLE, task_ex.state)
@@ -958,7 +1003,7 @@ class PoliciesTest(base.EngineTestCase):
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         self._assert_single_item(wf_ex.task_executions, name='task1')
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = self._assert_single_item(
@@ -979,7 +1024,7 @@ class PoliciesTest(base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wb.wf1', {})
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
         task_ex = self._assert_single_item(
@@ -989,9 +1034,7 @@ class PoliciesTest(base.EngineTestCase):
 
         self.assertEqual(states.SUCCESS, task_ex.state)
 
-        runtime_context = task_ex.runtime_context
-
-        self.assertEqual(4, runtime_context['concurrency'])
+        self.assertEqual(4, task_ex.runtime_context['concurrency'])
 
     def test_concurrency_is_in_runtime_context_from_var(self):
         wb_service.create_workbook_v2(CONCURRENCY_WB_FROM_VAR)
@@ -1000,12 +1043,13 @@ class PoliciesTest(base.EngineTestCase):
         wf_ex = self.engine.start_workflow('wb.wf1', {'concurrency': 4})
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
         task_ex = self._assert_single_item(
             wf_ex.task_executions,
             name='task1'
         )
 
-        self.assertEqual(4, task_ex.in_context['concurrency'])
+        self.assertEqual(4, task_ex.runtime_context['concurrency'])
 
     def test_wrong_policy_prop_type(self):
         wb = """---
@@ -1051,9 +1095,9 @@ class PoliciesTest(base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wf', {})
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         # Note: We need to reread execution to access related tasks.
-        wf_ex = db_api.get_execution(wf_ex.id)
+        wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(2, len(wf_ex.task_executions))
