@@ -39,6 +39,7 @@ name: wb1
 workflows:
   wf1:
     type: reverse
+
     input:
       - param1
       - param2
@@ -49,16 +50,17 @@ workflows:
       task1:
         action: std.echo output=<% $.param1 %>
         publish:
-          result1: <% $.task1 %>
+          result1: <% task(task1).result %>
 
       task2:
         action: std.echo output="'<% $.param1 %> & <% $.param2 %>'"
         publish:
-          final_result: <% $.task2 %>
+          final_result: <% task(task2).result %>
         requires: [task1]
 
   wf2:
     type: direct
+
     output:
       slogan: <% $.slogan %>
 
@@ -66,7 +68,7 @@ workflows:
       task1:
         workflow: wf1 param1='Bonnie' param2='Clyde' task_name='task2'
         publish:
-          slogan: "<% $.task1.final_result %> is a cool movie!"
+          slogan: "<% task(task1).result.final_result %> is a cool movie!"
 """
 
 WB2 = """
@@ -78,14 +80,17 @@ name: wb2
 workflows:
   wf1:
     type: direct
+
     tasks:
       task1:
         workflow: wf2
 
   wf2:
     type: direct
+
     output:
       var1: <% $.does_not_exist %>
+
     tasks:
         task1:
             action: std.noop
@@ -138,7 +143,7 @@ class SubworkflowsTest(base.EngineTestCase):
         )
 
         # Wait till workflow 'wf1' is completed.
-        self._await(lambda: self.is_execution_success(wf1_ex.id))
+        self.await_workflow_success(wf1_ex.id)
 
         wf1_ex = db_api.get_workflow_execution(wf1_ex.id)
 
@@ -148,7 +153,7 @@ class SubworkflowsTest(base.EngineTestCase):
         )
 
         # Wait till workflow 'wf2' is completed.
-        self._await(lambda: self.is_execution_success(wf2_ex.id))
+        self.await_workflow_success(wf2_ex.id, timeout=4)
 
         wf2_ex = db_api.get_workflow_execution(wf2_ex.id)
 
@@ -176,7 +181,7 @@ class SubworkflowsTest(base.EngineTestCase):
     @mock.patch.object(std_actions.EchoAction, 'run',
                        mock.MagicMock(side_effect=exc.ActionException))
     def test_subworkflow_error(self):
-        wf2_ex = self.engine.start_workflow('wb1.wf2', None)
+        self.engine.start_workflow('wb1.wf2', None)
 
         self._await(lambda: len(db_api.get_workflow_executions()) == 2, 0.5, 5)
 
@@ -188,26 +193,28 @@ class SubworkflowsTest(base.EngineTestCase):
         wf2_ex = self._assert_single_item(wf_execs, name='wb1.wf2')
 
         # Wait till workflow 'wf1' is completed.
-        self._await(lambda: self.is_execution_error(wf1_ex.id))
+        self.await_workflow_error(wf1_ex.id)
 
         # Wait till workflow 'wf2' is completed, its state must be ERROR.
-        self._await(lambda: self.is_execution_error(wf2_ex.id))
+        self.await_workflow_error(wf2_ex.id)
 
     def test_subworkflow_yaql_error(self):
         wf_ex = self.engine.start_workflow('wb2.wf1', None)
 
-        self._await(lambda: self.is_execution_error(wf_ex.id))
+        self.await_workflow_error(wf_ex.id)
 
         wf_execs = db_api.get_workflow_executions()
 
         self.assertEqual(2, len(wf_execs))
 
         wf2_ex = self._assert_single_item(wf_execs, name='wb2.wf2')
+
         self.assertEqual(states.ERROR, wf2_ex.state)
         self.assertIn('Can not evaluate YAQL expression', wf2_ex.state_info)
 
         # Ensure error message is bubbled up to the main workflow.
         wf1_ex = self._assert_single_item(wf_execs, name='wb2.wf1')
+
         self.assertEqual(states.ERROR, wf1_ex.state)
         self.assertIn('Can not evaluate YAQL expression', wf1_ex.state_info)
 
@@ -241,7 +248,7 @@ class SubworkflowsTest(base.EngineTestCase):
         self.assertDictContainsSubset(expected_start_params, wf1_ex.params)
 
         # Wait till workflow 'wf1' is completed.
-        self._await(lambda: self.is_execution_success(wf1_ex.id))
+        self.await_workflow_success(wf1_ex.id)
 
         # Wait till workflow 'wf2' is completed.
-        self._await(lambda: self.is_execution_success(wf2_ex.id))
+        self.await_workflow_success(wf2_ex.id)
