@@ -16,7 +16,6 @@ import mock
 from oslo_config import cfg
 
 from mistral.db.v2 import api as db_api
-from mistral.engine import default_engine as de
 from mistral import exceptions as exc
 from mistral.services import workbooks as wb_service
 from mistral.tests.unit.engine import base
@@ -139,7 +138,7 @@ workflows:
 
     tasks:
       task1:
-        action: std.echo output="Hi!"
+        action: std.echo output="Task 1"
         on-complete:
           - task3
           - pause
@@ -172,7 +171,7 @@ workflows:
           - pause
 
       task2:
-        action: std.mistral_http url="http://google.com"
+        action: std.async_noop
         # This one won't be finished when execution is already PAUSED.
         on-complete:
           - task4
@@ -198,7 +197,7 @@ class WorkflowResumeTest(base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wb.wf1', {})
 
-        self._await(lambda: self.is_execution_paused(wf_ex.id))
+        self.await_workflow_paused(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -211,7 +210,7 @@ class WorkflowResumeTest(base.EngineTestCase):
 
         self.assertEqual(2, len(wf_ex.task_executions))
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -243,7 +242,7 @@ class WorkflowResumeTest(base.EngineTestCase):
 
         self.assertEqual(states.RUNNING, wf_ex.state)
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
@@ -255,7 +254,7 @@ class WorkflowResumeTest(base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wb.wf1', {})
 
-        self._await(lambda: self.is_execution_paused(wf_ex.id))
+        self.await_workflow_paused(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -264,7 +263,7 @@ class WorkflowResumeTest(base.EngineTestCase):
 
         wf_ex = self.engine.resume_workflow(wf_ex.id)
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -279,7 +278,7 @@ class WorkflowResumeTest(base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wb.wf1', {})
 
-        self._await(lambda: self.is_execution_paused(wf_ex.id))
+        self.await_workflow_paused(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -289,17 +288,17 @@ class WorkflowResumeTest(base.EngineTestCase):
 
         # The exact number of tasks depends on which of two tasks
         # 'task1' and 'task2' completed earlier.
-        self.assertTrue(len(task_execs) >= 2)
+        self.assertGreaterEqual(len(task_execs), 2)
 
         task1_ex = self._assert_single_item(task_execs, name='task1')
         task2_ex = self._assert_single_item(task_execs, name='task2')
 
-        self._await(lambda: self.is_task_success(task1_ex.id))
-        self._await(lambda: self.is_task_success(task2_ex.id))
+        self.await_task_success(task1_ex.id)
+        self.await_task_success(task2_ex.id)
 
         self.engine.resume_workflow(wf_ex.id)
 
-        self._await(lambda: self.is_execution_success(wf_ex.id), 1, 5)
+        self.await_workflow_success(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -312,7 +311,7 @@ class WorkflowResumeTest(base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wb.wf1', {})
 
-        self._await(lambda: self.is_execution_paused(wf_ex.id))
+        self.await_workflow_paused(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -333,8 +332,9 @@ class WorkflowResumeTest(base.EngineTestCase):
 
         # Wait for task3 to be processed.
         task3_ex = self._assert_single_item(task_execs, name='task3')
-        self._await(lambda: self.is_task_success(task3_ex.id))
-        self._await(lambda: self.is_task_processed(task3_ex.id))
+
+        self.await_task_success(task3_ex.id)
+        self.await_task_processed(task3_ex.id)
 
         # Finish task2.
         task2_action_ex = db_api.get_action_executions(
@@ -343,42 +343,39 @@ class WorkflowResumeTest(base.EngineTestCase):
 
         self.engine.on_action_complete(task2_action_ex.id, utils.Result())
 
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+        self.await_workflow_success(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.SUCCESS, wf_ex.state, wf_ex.state_info)
         self.assertEqual(4, len(wf_ex.task_executions))
 
-    @mock.patch.object(de.DefaultEngine, '_fail_workflow')
-    def test_resume_fails(self, mock_fw):
+    def test_resume_fails(self):
         # Start and pause workflow.
         wb_service.create_workbook_v2(WORKBOOK_DIFFERENT_TASK_STATES)
 
         wf_ex = self.engine.start_workflow('wb.wf1', {})
 
-        self._await(lambda: self.is_execution_paused(wf_ex.id))
+        self.await_workflow_paused(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertEqual(states.PAUSED, wf_ex.state)
 
         # Simulate failure and check if it is handled.
-        err = exc.MistralException('foo')
+        err = exc.MistralError('foo')
 
         with mock.patch.object(
                 db_api,
-                'acquire_lock',
+                'get_workflow_execution',
                 side_effect=err):
 
             self.assertRaises(
-                exc.MistralException,
+                exc.MistralError,
                 self.engine.resume_workflow,
                 wf_ex.id
             )
 
-            mock_fw.assert_called_once_with(wf_ex.id, err)
-
     def test_resume_diff_env_vars(self):
         wb_service.create_workbook_v2(RESUME_WORKBOOK_DIFF_ENV_VAR)
 
@@ -391,7 +388,7 @@ class WorkflowResumeTest(base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow('wb.wf1', {}, env=env)
 
-        self._await(lambda: self.is_execution_paused(wf_ex.id))
+        self.await_workflow_paused(wf_ex.id)
 
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -420,7 +417,9 @@ class WorkflowResumeTest(base.EngineTestCase):
 
         # Update the env variables and resume workflow.
         self.engine.resume_workflow(wf_ex.id, env=updated_env)
-        self._await(lambda: self.is_execution_success(wf_ex.id))
+
+        self.await_workflow_success(wf_ex.id)
+
         wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
         self.assertDictEqual(updated_env, wf_ex.params['env'])
@@ -435,7 +434,12 @@ class WorkflowResumeTest(base.EngineTestCase):
 
         self.assertEqual(states.SUCCESS, task_2_ex.state)
 
-        task_2_result = data_flow.get_task_execution_result(task_2_ex)
+        # Re-read task execution, otherwise lazy loading of action executions
+        # may not work.
+        with db_api.transaction():
+            task_2_ex = db_api.get_task_execution(task_2_ex.id)
+
+            task_2_result = data_flow.get_task_execution_result(task_2_ex)
 
         self.assertEqual(updated_env['var1'], task_2_result)
 
@@ -447,6 +451,11 @@ class WorkflowResumeTest(base.EngineTestCase):
 
         self.assertEqual(states.SUCCESS, task_3_ex.state)
 
-        task_3_result = data_flow.get_task_execution_result(task_3_ex)
+        # Re-read task execution, otherwise lazy loading of action executions
+        # may not work.
+        with db_api.transaction():
+            task_3_ex = db_api.get_task_execution(task_3_ex.id)
+
+            task_3_result = data_flow.get_task_execution_result(task_3_ex)
 
         self.assertEqual(updated_env['var2'], task_3_result)
