@@ -16,8 +16,11 @@
 
 import os
 
+from tempest import config
 from tempest.test_discover import plugins
 
+from mistral_tempest_tests import config as mistral_config
+
 
 class MistralTempestPlugin(plugins.TempestPlugin):
     def load_tests(self):
@@ -28,7 +31,11 @@ class MistralTempestPlugin(plugins.TempestPlugin):
         return full_test_dir, base_path
 
     def register_opts(self, conf):
-        pass
+        config.register_opt_group(
+            conf,
+            mistral_config.service_available_group,
+            mistral_config.ServiceAvailableGroup
+        )
 
     def get_opt_lists(self):
         pass
